#!/bin/sh

library(ncdf4)
library(raster)

# ncfiles holding the decadal average data of the 9 variables of interest

MyDataHome <- c("SET_PATH_TO_WHERE_YOU_HAVE_STORED_THE_NC-FILES!!!")		# replace this with your own personal path to where you store the nc-files

path1 <- c(paste(MyDataHome,"RCP85_equi_wifi_decadal_avg.nc",sep=""))
path2 <- c(paste(MyDataHome,"RCP85_trans_wifi_decadal_avg.nc",sep=""))
path3 <- c(paste(MyDataHome,"RCP45_equi_wifi_decadal_avg.nc",sep=""))
path4 <- c(paste(MyDataHome,"RCP45_trans_wifi_decadal_avg.nc",sep=""))
path5 <- c(paste(MyDataHome,"RCP85_equi_nofi_decadal_avg.nc",sep=""))
path6 <- c(paste(MyDataHome,"RCP85_trans_nofi_decadal_avg.nc",sep=""))
path7 <- c(paste(MyDataHome,"RCP45_equi_nofi_decadal_avg.nc",sep=""))
path8 <- c(paste(MyDataHome,"RCP45_trans_nofi_decadal_avg.nc",sep=""))
pvec <- c(path1,path2,path3,path4,path5,path6,path7,path8)

varnames <- c("Savanna tree cover","Forest tree cover","Aboveground tree biomass","C3:C4 grass ratio","Mean tree height","Maximum tree height","Number of tree individuals","Aboveground grass biomass","Total tree cover")
dec_vec <- c("1970-1979","1980-1989","1990-1999","2000-2009","2010-2019","2020-2029","2030-2039","2040-2049","2050-2059","2060-2069","2070-2079","2080-2089","2090-2099")
biome_labels <- c("Desert","C3 grassland","C4 grassland","C3 Savanna","C4 Savanna","Woodland","Forest")
scenario <- c("8.5wifi_e","8.5wifi_t","4.5wifi_e","4.5_wifi_t","8.5nofi_e","8.5nofi_t","4.5nofi_e","4.5_nofi_t")
dvs <- c("1970s","1980s","1990s","2000s","2010s","2020s","2030s","2040s","2050s","2060s","2070s","2080s","2090s")
ScLab <- c("E_8.5_wifi","T_8.5_wifi","E_4.5_wifi","T_4.5_wifi","E_8.5_nofi","T_8.5_nofi","E_4.5_nofi","T_4.5_nofi")

#---------------------------------------------------------------------------------------
# All nc-files have the same internal structure. Open one to acquire the dimensions

message("Acquiring nc-file's dimensions")

ncid <- nc_open(path1)
lons <- ncvar_get(ncid,"lon")
lats <- ncvar_get(ncid,"lat")
time <- ncvar_get(ncid,"time")
nlon <- dim(lons)						# number of longitude values
nlat <- dim(lats)						# number of latitude values
ntime <- dim(time)						# number of time slices (decades)

var1 <- raster(ncvar_get(ncid,"Savanna tree cover",start=c(1,1,1), count=c(nlon,nlat,1))) 
var1 <- setExtent(var1,c(min(lons),max(lons),min(lats),max(lats)))
cellarea <- as.matrix(area(var1))
nc_close(ncid)
nvar <- 9								# number of variables
nscen <- 8								# number of scenarios
nbiome <- 7								# number of biome categories

wf85 <- 1
wf45 <- 2
nf85 <- 3
nf45 <-4

#---------------------------------------------------------------------------------------
# get variables from nc-files holding decadal average values for the 9 variables of interest

message("Reading variables from nc-files")
vars <- array(NA,c(nlon,nlat,nvar,nscen,ntime))

for(n in 1:nscen)
{
	ncid <- nc_open(pvec[n])
	for(v in 1:nvar)
	{
		vars[,,v,n,] <- ncvar_get(ncid,varnames[v])
	}
	nc_close(ncid)
}

#---------------------------------------------------------------------------------------

# Biomization based on the criteria listed in Supplementary Table S1 
# Biome types are: Desert (1), C3-Grassland (2), C4-Grassland (3), C3-Savanna (4), C4-Savanna (5), Woodland (6) and Forest (7)
# biome_labels <- c("Desert","C3-grassland","C4-grassland","C3-Savanna","C4-Savanna","Woodland","Forest")
# 						1		2				3				4			5			6			7

message("Conducting biome classification")

BT <- array(NA,c(nlon,nlat,nscen,ntime))		# Biome type array

for(t in 1:ntime)
{
 	for(n in 1:nscen)
 	{
 		for(x in 1:nlon)
 		{
 			for(y in 1:nlat)
 			{
	 			if(!anyNA(vars[x,y,c(1,2,4,8,9),n,t]))
	 			{
	 				# desert
	 				if(vars[x,y,8,n,t] <= 0.5 && vars[x,y,9,n,t] <= 0.1) { BT[x,y,n,t] <- 1}	# grass less than 0.5 t/ha and tree coyer <= 10 %
	 				# grassland
	 				if(vars[x,y,8,n,t] > 0.5 && vars[x,y,9,n,t] <= 0.1 && vars[x,y,4,n,t] > 0.5) {BT[x,y,n,t] <- 2}		# C3 grassland
	 				if(vars[x,y,8,n,t] > 0.5 && vars[x,y,9,n,t] <= 0.1 && vars[x,y,4,n,t] <= 0.5) {BT[x,y,n,t] <- 3}	# C4 grassland
	 				# sayanna
	 				if(vars[x,y,9,n,t] > 0.1 && vars[x,y,9,n,t] <= 0.8 && vars[x,y,2,n,t] <= vars[x,y,1,n,t] && vars[x,y,4,n,t] > 0.5) {BT[x,y,n,t] <- 4}	# C3 sayanna
	 				if(vars[x,y,9,n,t] > 0.1 && vars[x,y,9,n,t] <= 0.8 && vars[x,y,2,n,t] <= vars[x,y,1,n,t] && vars[x,y,4,n,t] <= 0.5) {BT[x,y,n,t] <- 5}	# C4 sayanna
	 				# woodland
	 				if(vars[x,y,9,n,t] > 0.1 && vars[x,y,9,n,t] <= 0.8 && vars[x,y,2,n,t] > vars[x,y,1,n,t]) {BT[x,y,n,t] <- 6}	# woodland
	 				# forest
	 				if(vars[x,y,9,n,t] > 0.8) {BT[x,y,n,t] <- 7}	# forest
	 			}
	 		}	
 		}
 	}
}

#---------------------------------------------------------------------------------------

# calculate each biome's total area for all scenarios and time slices

message("Calculating biome areas")

BA <- array(0,c(nbiome,nscen,ntime))		# summed biome areas, in km2
BAF <- array(0,c(nbiome,nscen,ntime))		# fraction of total Africa occupied by a given biome, at a given time, in a given scenario
totarea <- array(0,c(nscen,ntime))			# total area occupied by all biomes

for(t in 1:ntime)
{
	for(n in 1:nscen)
	{
		for(x in 1:nlon)
		{
			for(y in 1:nlat)
			{
				for(b in 1:nbiome)
				{
					if(!is.na(BT[x,y,n,t]))
					{
						if(BT[x,y,n,t]==b) { BA[b,n,t] <- BA[b,n,t] + cellarea[x,y] }
					}	
				}
			}
		}		
		totarea[n,t] <- sum(BA[,n,t])
		BAF[,n,t] <- BA[,n,t] / totarea[n,t]
	}
}

#---------------------------------------------------------------------------------------------
# how many biome types are there over time, within a given scenario?

message("Determining number of biome types over time")

No_BT <- array(NA,c(nlon,nlat,nscen))

for(n in 1:nscen)
{
	for(x in 1:nlon)
	{
		for(y in 1:nlat)
		{
			if(!anyNA(BT[x,y,n,]))
			{
				No_BT[x,y,n] <- length(unique(BT[x,y,n, ]))
			}	
		}
	}
}

#---------------------------------------------------------------------------------------------
# how many biome changes happen over time, within a given scenario?

message("Determining number of biome changes over time")

No_BT_changes <- array(NA,c(nlon,nlat,nscen))

for(n in 1:nscen)
{
	for(x in 1:nlon)
	{
		for(y in 1:nlat)
		{
			if(!anyNA(BT[x,y,n,]))
			{
				counter <- 0
				ident <- BT[x,y,n,1]
				{
					for(t in 1:ntime)
					{
						if(ident != BT[x,y,n,t])
						{
							counter <- counter + 1
							ident <- BT[x,y,n,t]
						}
					}
				}
				No_BT_changes[x,y,n] <- counter
			}	
		}
	}
}

#---------------------------------------------------------------------------------------------
# Number of biome changes over divided by number of biome types (indicates "flickering behavior"; the higher the ratio, the more flickering between a subset of biome types)

message("Determining range of biome changes to biome types over time")

Ratio_BT <- (No_BT_changes) / No_BT

#---------------------------------------------------------------------------------------------
# for a given decade, also mark how many different biome types a given pixel has had so far (1 - 7 are possible)

message("Determining number of biome types over time, for each decade")

No_BTs_cum <- array(NA,c(nlon,nlat,nscen,ntime))

for(n in 1:nscen)
{
	for(x in 1:nlon)
	{
		for(y in 1:nlat)
		{
			if(!anyNA(BT[x,y,n,]))
			{
				counter <- 1
				for(t in 1:ntime)
				{
					ident <- BT[x,y,n,t]
					if(!ident %in% BT[x,y,n,(1:(t-1))] )	# check if that ident has occurred alreadx up to time t-1; if not, add it
					{
						ident <- BT[x,y,n,t]
						counter <- counter + 1
					}
					No_BTs_cum[x,y,n,t] <- counter
				}
			}	
		}
	}
}

#---------------------------------------------------------------------------------------------
# for a given decade, mark how many biome changes it has already experienced (0 - 12 are possible)

message("Determining number of biome changes over time, for each decade")

No_BTchanges_cum <- array(NA,c(nlon,nlat,nscen,ntime))

for(n in 1:nscen)
{
	for(x in 1:nlon)
	{
		for(y in 1:nlat)
		{
			if(!anyNA(BT[x,y,n,]))
			{
				counter <- 0
				ident <- BT[x,y,n,1]
				for(t in 1:ntime)
				{
					if(ident != BT[x,y,n,t])
					{
						ident <- BT[x,y,n,t]
						counter <- counter + 1
					}
					No_BTchanges_cum[x,y,n,t] <- counter
				}
			}	
		}
	}
}

#---------------------------------------------------------------------------------------------
# for a given decade, calculate the ratio between biome changes experienced up to that decade and number of biome types up to that decade

message("Determining ratio of biome changes to biome types, for each decade")

Ratio_cum <- No_BTchanges_cum / No_BTs_cum

#------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# compare where biome types agree between scenarios and where they differ; mask out areas of agreement as white, only show color for areas where disagreement exists => two sets of maps next to one another

# transient against equilibrium (n1,n2)
# fire against no fire (o1,o2)
# RCP8.5 against RCP4.5 (p1,p2)

message("Comaring biome type agreement between different scenarios")

Bdiff_T_E <- array(NA,c(nlon,nlat,(nscen/2),ntime))
Bdiff_wifinofi <- array(NA,c(nlon,nlat,(nscen/2),ntime))
Bdiff_85_45 <- array(NA,c(nlon,nlat,(nscen/2),ntime))

p1 <- c(1,2,5,6)
p2 <- c(3,4,7,8)

for(n in 1:(nscen/2))
{
	n1 <- n*2-1
	n2 <- n*2
	o1 <- n
	o2 <- n+4
	for(t in 1:ntime)
	{
		for(y in 1:nlat)
		{
			for(x in 1:nlon)
			{
				if(!anyNA(BT[x,y,,t]))
				{
					if(BT[x,y,n1,t] != BT[x,y,n2,t])
					{
						Bdiff_T_E[x,y,n,t] <- 1
					}
					if(BT[x,y,o1,t] != BT[x,y,o2,t])
					{
						Bdiff_wifinofi[x,y,n,t] <- 1
					}
					if(BT[x,y,p1[n],t] != BT[x,y,p2[n],t])
					{
						Bdiff_85_45[x,y,n,t] <- 1
					}			
				}
			}
		}
	}
}

#-------------------------------------------------------------------------------------------------------------
# areas of least concern based on biomes: biomes that are stable over time

message("Determining time-stable biome types for each scenario")

time_stable_BT <- array(NA,c(nlon,nlat,nscen))	# pixels that have only one biome type over time (non-NA)

for(n in 1:nscen)
{
	for(x in 1:nlon)
	{
		for(y in 1:nlat)
		{
			if(!anyNA(BT[x,y,n,]))
			{
				if(length(unique(BT[x,y,n,]))==1)
				{
					time_stable_BT[x,y,n] <- BT[x,y,n,1]
				}
			}
		}
	}
}

message("Determining biome stability accross all time slices and scenarios")

most_stable_BT <- array(NA,c(nlon,nlat))	# pixels that have only one biome type over time (non-NA) and accross all scenarios

for(x in 1:nlon)
{
	for(y in 1:nlat)
	{
		if(!anyNA(time_stable_BT[x,y,]))
		{
			if(length(unique(time_stable_BT[x,y,]))==1)
			{
				most_stable_BT[x,y] <- time_stable_BT[x,y,1]
			}
		}
	}
}


