#!/bin/sh

library(IDPmisc)		# for function NaRV.omit

MyHomeDirectory <- c("SET_PATH_TO_WHERE_YOU_HAVE_STORED_THE_BIOME_ANALYSIS_SCRIPT!!!")		# put the path to your own directory down here
BiomeAnalysis <- paste(MyHomeDirectory,"Biome_analysis.R",sep="")
source(BiomeAnalysis)		# variables are read in from nc-files in Biome_analysis.R and biomes are determined based on variables

#---------------------------------------------------------------------------------------
# spatially averaged variables, per scenario, per time slice

message("Calculating spatial averages of variables by scenario and decade")

vars_savg <- array(0,c(nvar,nscen,ntime))

Afri_area <- array(0,c(nvar,nscen,ntime))			# area of Africa (only counting where valid variable values exist)

for(t in 1:ntime)
{
	for(n in 1:nscen)
	{
		for(x in 1:nlon)
		{
			for(y in 1:nlat)
			{
				if(!anyNA(vars[x,y, ,n,t]))
				{
					for(v in 1:nvar)
					{
						vars_savg[v,n,t] <- vars_savg[v,n,t] + (vars[x,y,v,n,t] * cellarea[x,y])
						Afri_area[v,n,t] <- Afri_area[v,n,t] + cellarea[x,y]
					}
				}
			}	
		}
		vars_savg[,n,t] <- vars_savg[,n,t] / Afri_area[,n,t]
	}
}

#---------------------------------------------------------------------------------------
# consensus desert: find all pixels that are desert in all 8 possible scenarios, at all times, (to use this as a mask later on to avoid spurious results from desert areas)

message("Identify all grid cells that are consistently desert")

desert_mask <- array(NA,c(nlon,nlat))

for(x in 1:nlon)
{
	for(y in 1:nlat)
	{
		if(!anyNA(BT[x,y,,]))
		{
			if(sum(BT[x,y,,])==(nscen*ntime)) {desert_mask[x,y] <- -9999}		# all entries == 1, i.e., always desert
		}	
	}
}

#---------------------------------------------------------------------------------------
# to mask out desert, set all pixels that are exclusively desert = NA; 
# mask desert from further data tweaking efforts (can very easily be commented out as section here to consider all data points including desert)

message("Mask out all grid cells that are consistently desert")

for(t in 1:ntime)
{
	for(n in 1:nscen)
	{
		for(x in 1:nlon)
		{
			for(y in 1:nlat)
			{
				for(v in 1:nvar)
				{
					if(!is.na(desert_mask[x,y]))
					{
						vars[x,y,v,n,t] <- NA
					}
				}
			}
		}
	}
}

#---------------------------------------------------------------------------------------
# standardize variables to mean = 0 and stdev = 1 to make variables comparable wit one another

message("Standardizing variables to mean=0 and stdev=2")

std_vars <- array(0,c(nlon,nlat,nvar,nscen,ntime))	# array holding the standardized variable values

for(v in 1:nvar)
{
	varmean <- mean(na.omit(as.vector(vars[,,v,,])))
	varsd <- sd(na.omit(as.vector(vars[,,v,,])))
	
	std_vars[,,v,,] <- (vars[,,v,,]-varmean)/varsd
}

#---------------------------------------------------------------------------------------
# calculate the Euclidean distance between same-same decade pairs of congruent scenarios

message("Calculating Euclidean distance between SDPs")

edist <- array(NA,c(nlon,nlat,(nscen/2),ntime))

for(t in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		for(x in 1:nlon)
		{
			for(y in 1:nlat)
			{
				n1 <- n*2-1				# all uneven indices, i.e., all equilibrium scenarios
				n2 <- n*2				# all even indices, i.e., all transient scenarios
				edist[x,y,n,t] <- dist(rbind(std_vars[x,y, ,n1,t], std_vars[x,y, ,n2,t]), method="euclidean")
			}
		}
	}	
}

#---------------------------------------------------------------------------------------
# Difference in Euclidean distance between scenarios, based on quotient ratio of scenarios
# a) wifi-nofi comparison (wf85-nf85; wf45-nf45)
# b) inter-scenario comparison (wf85-wf45; nf85-nf45)

message("Determining ratio of Euclidean distance between different scenarios")

div_edist <- array(NA,c(nlon,nlat,4,ntime))		# (pos1: wf85-nf85; pos2: wf45-nf45; pos3: wf85-wf45; pos4: nf85-nf45)

for(t in 1:ntime)
{
	for(x in 1:nlon)
	{
		for(y in 1:nlat)
		{
			if(!anyNA(edist[x,y,,t]))
			{
				if(edist[x,y,wf85,t] >= edist[x,y,nf85,t]) { div_edist[x,y,1,t] <- edist[x,y,wf85,t] / edist[x,y,nf85,t] }			
				if(edist[x,y,wf85,t] <  edist[x,y,nf85,t]) { div_edist[x,y,1,t] <- edist[x,y,nf85,t] / edist[x,y,wf85,t] * -1. }
				if(edist[x,y,wf45,t] >= edist[x,y,nf45,t]) { div_edist[x,y,2,t] <- edist[x,y,wf45,t] / edist[x,y,nf45,t] }
				if(edist[x,y,wf45,t] <  edist[x,y,nf45,t]) { div_edist[x,y,2,t] <- edist[x,y,nf45,t] / edist[x,y,wf45,t] * -1. }
				if(edist[x,y,wf85,t] >= edist[x,y,wf45,t]) { div_edist[x,y,3,t] <- edist[x,y,wf85,t] / edist[x,y,wf45,t] }
				if(edist[x,y,wf85,t] <  edist[x,y,wf45,t]) { div_edist[x,y,3,t] <- edist[x,y,wf45,t] / edist[x,y,wf85,t] * -1. }
				if(edist[x,y,nf85,t] >= edist[x,y,nf45,t]) { div_edist[x,y,4,t] <- edist[x,y,nf85,t] / edist[x,y,nf45,t] }
				if(edist[x,y,nf85,t] <  edist[x,y,nf45,t]) { div_edist[x,y,4,t] <- edist[x,y,nf45,t] / edist[x,y,nf85,t] * -1. }
			}
		}
	}
}

#---------------------------------------------------------------------------------------
# calculate the spatially-averaged (across all Africa) Euclidean distance and its standard deviation (for each timeslice, for each scenario pair)

message("Calculating spatially averaged Euclidean distance for SDPs, for each scenario and decade")

smean_edist <- array(NA,c((nscen/2),ntime))		# spatial-mean Euclidean distance
ssd_edist <- array(NA,c((nscen/2),ntime))		# standard deviation of spatial-mean Euclidean distance

for(t in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		smean_edist[n,t] <- mean(NaRV.omit(as.vector(edist[,,n,t])))
		ssd_edist[n,t] <- sd(NaRV.omit(as.vector(edist[,,n,t])))
	}
}

#---------------------------------------------------------------------------------------
# for each grid cell, find the minimum and maximum Euclidean distance and the corresponding decade to go with the min/max value

message("Determining minimum and maximum Euclidean distance and corresponding decades")

numstats <- 4		#  mindec, maxdec, minval, maxval
stats_edist <- array(NA,c(nlon,nlat,nscen/2,numstats))		

for(n in 1:(nscen/2))
{
	for(x in 1:nlon)
	{
		for(y in 1:nlat)
		{
			if(!anyNA(edist[x,y,n, ]))
			{
				stats_edist[x,y,n,1] <- which.min(edist[x,y,n, ])				# minimum decade
				stats_edist[x,y,n,2] <- which.max(edist[x,y,n, ])				# mayimum decade
				stats_edist[x,y,n,3] <- edist[x,y,n,stats_edist[x,y,n,1]]			# distance value of minimum decade
				stats_edist[x,y,n,4] <- edist[x,y,n,stats_edist[x,y,n,2]]			# distance value of mayimum decade
			}	
		} 
	}
}

#---------------------------------------------------------------------------------------------------------------------
# for each transient-scenario grid cell, find the decade that is most similar in the equilibrium conditions scenario, and give back the distance value for the decade pairing

message("Finding closest decade partners and determining Euclidean distance between CDPs")

EuAll <- array(NA,c(nlon,nlat,(nscen/2),ntime,ntime))		# Euclidean distance based on standardized variables for all decadal combinations; first time dimension for transient-year values, second time dimension for equilibrium-year values

for(tt in 1:ntime)				# loop through the transient years
{
	for(n in 1:(nscen/2))
	{
		n1 <- n*2-1				# all uneven indices, i.e., all equilibrium scenarios
		n2 <- n*2				# all even Indices, i.e., all transient scenarios
		
		message("Working on transient decade ",tt)
		for(te in 1:ntime)			# loop through the equilibrium years
		{
			for(x in 1:nlon)
			{
				for(y in 1:nlat)
				{
					EuAll[x,y,n,tt,te] <- dist(rbind(std_vars[x,y, ,n2,tt], std_vars[x,y, ,n1,te]), method="euclidean")
				}
			}
		}
	}	
}

MinDec_EuPast <- array(NA,c(nlon,nlat,(nscen/2),2,ntime))		# Dim1: Index of equilibrium year that is closest to transient year; Dim2: actual value of closest year

for(dt in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		for(x in 1:nlon)
		{
			for(y in 1:nlat)
			{
				if(!anyNA(EuAll[x,y,n,dt, ]))
				{
					# Minimum Euclidean Distance
					MinDec_EuPast[x,y,n,1,dt] <- which.min(EuAll[x,y,n,dt,1:dt])
					MinDec_EuPast[x,y,n,2,dt] <- EuAll[x,y,n,dt,(MinDec_EuPast[x,y,n,1,dt])]		
				}					
			}
		}
	}	
}

#---------------------------------------------------------------------------------------------------------------------
# Difference in Euclidean distance (closest decade partners) between scenarios, based on quotient ratio between scenarios under comparison
# a) wifi-nofi comparison (wf85-nf85; wf45-nf45)
# b) inter-scenario comparison (wf85-wf45; nf85-nf45)

message("Determining ratio of Euclidean distance between CDPs")

div_edist_cross <- array(NA,c(nlon,nlat,4,ntime))		# (pos1: wf85-nf85; pos2: wf45-nf45; pos3: wf85-wf45; pos4: nf85-nf45)

for(t in 1:ntime)
{
	for(x in 1:nlon)
	{
		for(y in 1:nlat)
		{
			if(!anyNA(MinDec_EuPast[x,y,,2,t]))
			{
				if(MinDec_EuPast[x,y,wf85,2,t] >= MinDec_EuPast[x,y,nf85,2,t]) { div_edist_cross[x,y,1,t] <- MinDec_EuPast[x,y,wf85,2,t] / MinDec_EuPast[x,y,nf85,2,t] }			
				if(MinDec_EuPast[x,y,wf85,2,t] <  MinDec_EuPast[x,y,nf85,2,t]) { div_edist_cross[x,y,1,t] <- MinDec_EuPast[x,y,nf85,2,t] / MinDec_EuPast[x,y,wf85,2,t] * -1. }
				if(MinDec_EuPast[x,y,wf45,2,t] >= MinDec_EuPast[x,y,nf45,2,t]) { div_edist_cross[x,y,2,t] <- MinDec_EuPast[x,y,wf45,2,t] / MinDec_EuPast[x,y,nf45,2,t] }
				if(MinDec_EuPast[x,y,wf45,2,t] <  MinDec_EuPast[x,y,nf45,2,t]) { div_edist_cross[x,y,2,t] <- MinDec_EuPast[x,y,nf45,2,t] / MinDec_EuPast[x,y,wf45,2,t] * -1. }
				if(MinDec_EuPast[x,y,wf85,2,t] >= MinDec_EuPast[x,y,wf45,2,t]) { div_edist_cross[x,y,3,t] <- MinDec_EuPast[x,y,wf85,2,t] / MinDec_EuPast[x,y,wf45,2,t] }
				if(MinDec_EuPast[x,y,wf85,2,t] <  MinDec_EuPast[x,y,wf45,2,t]) { div_edist_cross[x,y,3,t] <- MinDec_EuPast[x,y,wf45,2,t] / MinDec_EuPast[x,y,wf85,2,t] * -1. }
				if(MinDec_EuPast[x,y,nf85,2,t] >= MinDec_EuPast[x,y,nf45,2,t]) { div_edist_cross[x,y,4,t] <- MinDec_EuPast[x,y,nf85,2,t] / MinDec_EuPast[x,y,nf45,2,t] }
				if(MinDec_EuPast[x,y,nf85,2,t] <  MinDec_EuPast[x,y,nf45,2,t]) { div_edist_cross[x,y,4,t] <- MinDec_EuPast[x,y,nf45,2,t] / MinDec_EuPast[x,y,nf85,2,t] * -1. }
			}
		}
	}
}

#---------------------------------------------------------------------------------------------------------------------
# calculate spatially-averaged Euclidean distance for crosswise-decade-pairing, and spatial standard deviation, for each transient decade

message("Calculating spatially averaged Euclidean distance for CDPs")

smean_edist_cross <- array(NA,c((nscen/2),ntime))		# spatial-mean Euclidean distance
ssd_edist_cross <- array(NA,c((nscen/2),ntime))		# standard deviation of spatial-mean Euclidean distance

for(t in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		smean_edist_cross[n,t] <- mean(NaRV.omit(as.vector(MinDec_EuPast[,,n,2,t])))
		ssd_edist_cross[n,t] <- sd(NaRV.omit(as.vector(MinDec_EuPast[,,n,2,t])))
	}
}

#---------------------------------------------------------------------------------------------------------------------
# show absolute lag duration (in decades)

message("Determining lag time between CDPs")

Lagtime_EuPast  <- array(NA,c(nlon,nlat,(nscen/2),ntime))

for(dt in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		for(x in 1:nlon)
		{
			for(y in 1:nlat)
			{
				Lagtime_EuPast[x,y,n,dt] <- dt-MinDec_EuPast[x,y,n,1,dt]			
			}
		}
	}	
}

#---------------------------------------------------------------------------------------------------------------------
# Absolute difference in lagtime between scenarios
# a) wifi-nofi comparison (wf85-nf85; wf45-nf45)
# b) inter-scenario comparison (wf85-wf45; nf85-nf45)

message("Comparing scenarios with respect to lag times")

diff_lagtime <- array(NA,c(nlon,nlat,4,ntime))		# (pos1: wf85-nf85; pos2: wf45-nf45; pos3: wf85-wf45; pos4: nf85-nf45)

for(t in 1:ntime)
{
	for(x in 1:nlon)
	{
		for(y in 1:nlat)
		{
			if(!anyNA(Lagtime_EuPast[x,y,,t]))
			{
				diff_lagtime[x,y,1,t] <- Lagtime_EuPast[x,y,wf85,t] - Lagtime_EuPast[x,y,nf85,t] 			 
				diff_lagtime[x,y,2,t] <- Lagtime_EuPast[x,y,wf45,t] - Lagtime_EuPast[x,y,nf45,t] 
				diff_lagtime[x,y,3,t] <- Lagtime_EuPast[x,y,wf85,t] - Lagtime_EuPast[x,y,wf45,t] 
				diff_lagtime[x,y,4,t] <- Lagtime_EuPast[x,y,nf85,t] - Lagtime_EuPast[x,y,nf45,t] 
			}
		}
	}
}

#---------------------------------------------------------------------------------------------------------------------
# calculate the spatially-averaged lagtime for each decade and scenario, and the spatial standard deviation

message("Calculating spatially averaged lag time for each decade and scenario")

smean_lagtime <- array(NA,c((nscen/2),ntime))		# spatial-mean Euclidean distance
ssd_lagtime <- array(NA,c((nscen/2),ntime))		# standard deviation of spatial-mean Euclidean distance

for(t in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		smean_lagtime[n,t] <- mean(NaRV.omit(as.vector(Lagtime_EuPast[,,n,t])))
		ssd_lagtime[n,t] <- sd(NaRV.omit(as.vector(Lagtime_EuPast[,,n,t])))
	}
}

#---------------------------------------------------------------------------------------------------------------------

# find out, for each grid pixel and decade, which variable contributes most strongly to the dissimiliarity/deviance for the Euclidean metric between SDPs
# doing this as a "bootstrapping" approach, i.e., set each feature's distance to 0, one at a time, and compare by how much 
# this changes the outcome from the result with all features being acknowledged (e.g., based on percentage deviance from the "full value" in case of Euclidean distance)

# pairwise decades (same index equilibrium and transient)

# do a permutation in which, one at a time, equilibrium and transient variable are set equal to one another, i.e., the difference between both = 0, and see how much the resulting Euclidean
# distance deviates from the one calculated from the full set.

message("Determining variable contributions to full Euclidean distance between SDPs - takes a little while")

edist_perm <- array(NA,c(nlon,nlat,nvar,(nscen/2),ntime))		# prozentuale Abweichung vom "vollen Wert" verursacht durch die betreffende Variable

for(t in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		n1 <- n*2-1				# alle ungeraden Indices, i.e., all equilibrium scenarios
		n2 <- n*2				# alle geraden Indices, i.e., all transient scenarios
	
		for(x in 1:nlon)
		{
			for(y in 1:nlat)
			{
				if(!anyNA(std_vars[x,y, ,,t]))
				{			
					for(v in 1:nvar)
					{
						tmp <- std_vars[x,y, ,n1,t]		# tmp: vector holding all 9 standardized vars 
						
						tmp[v] <- std_vars[x,y,v,n2,t]		# replace the equilibrium variable under consideration with the transient variable under consideration, while leaving all other variables at their equilibrium values
						
						edist_perm[x,y,v,n,t]  <- (edist[x,y,n,t] - dist(rbind(tmp, std_vars[x,y, ,n2,t]), method="euclidean"))/edist[x,y,n,t] * 100.
			
					}
				}
			}
		}
	}	
}

#---------------------------------------------------------------------------------------------------------------------
# sort the variables by rank (as far as possible)

message("Sorting variables according to contribution to full Euclidean distance between SDPs - takes a little while")

sort_edist_vars <- array(NA,c(nlon,nlat,nvar,(nscen/2),ntime))		# sort variables in descending order according to the percent deviance its omittance causes from the "full" distance value
spdiff_edist_vars <- array(NA,c(nlon,nlat,nvar,(nscen/2),ntime))	# percentage deviation from "full" Euclidean distance caused by variable v

for(t in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		for(x in 1:nlon)
		{
			for(y in 1:nlat)
			{	
				if(!anyNA(edist_perm[x,y, ,n,t]))
				{
					for(r in 1:nvar)
						{
							idya <- which(edist_perm[x,y, ,n,t]==(sort(edist_perm[x,y, ,n,t],TRUE)[r]))			# it may happen that some vars are equal in rank (usually the ones that = 0), therefore need this restriction
							if(length(idya)==1) sort_edist_vars[x,y,r,n,t] <- idya							
						}
				}
			}
		}
	}	
}

for(t in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		for(x in 1:nlon)
		{
			for(y in 1:nlat)
			 {
			 	for(v in 1:nvar)
			 	{
				 	idxs <- sort_edist_vars[x,y,v,n,t]	# this is the variable that corresponds to the current rank, and the idx required to get the pdiff-value that goes with it
				 	if(!is.nan(idxs)) spdiff_edist_vars[x,y,v,n,t] <- edist_perm[x,y,idxs,n,t]
			 	}		 	
			 }
		}
	}	
}

#---------------------------------------------------------------------------------------------------------------------
# count the number of pixels that each variable "dominates" for a given rank level (not considering how strongly it dominates them in this step), for the SDPs

message("Determining number of grid pixels dominated by a given variable")

level <- 1
sum_dom_vars <- array(0,c(nvar,(nscen/2),ntime))		# fraction of total African area "dominated" by a given variable, at a given time, for a given scenario

for(t in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		for(v in 1:nvar)
		{
			for(x in 1:nlon)
			{
				for(y in 1:nlat)
				{
					if(!is.na(sort_edist_vars[x,y,level,n,t]))
					{
						if(sort_edist_vars[x,y,level,n,t]==v) { sum_dom_vars[v,n,t] <- sum_dom_vars[v,n,t] + cellarea[x,y] }		
					}	
				}
			}
			sum_dom_vars[v,n,t] <- sum_dom_vars[v,n,t] / totarea[n,t] 	
		}
	}	
}

#---------------------------------------------------------------------------------------------------------------------
# variable contribution to full Euclidean distance between SDPs as percent deviance from total Euclidean distance:

message("Calculating average percent deviance from full Euclidean distance caused by dominant variables in SDPs")

nlevel <- nvar

Avg_PDomVar <- array(0,c(nvar,nlevel,(nscen/2),ntime))
SD_PDomVar <- array(0,c(nvar,nlevel,(nscen/2),ntime))
AAvg_PDomVar <- array(0,c(nlevel,(nscen/2),ntime))
ASD_PDomVar <- array(0,c(nlevel,(nscen/2),ntime))

for(level in 1:nvar)
{
	for(t in 1:ntime)
	{
		for(n in 1:(nscen/2))
		{
			for(v in 1:nvar)
			{
				h <- NULL				# make sure to re-set this helper for each var
				for(x in 1:nlon)
				{
					mask <- which(sort_edist_vars[x,,level,n,t]==v)
					h <- c(h,spdiff_edist_vars[x,mask,level,n,t])
				}
				Avg_PDomVar[v,level,n,t] <- mean(h)
				SD_PDomVar[v,level,n,t] <- sd(h)
			}
			AAvg_PDomVar[level,n,t] <- mean(NaRV.omit(as.vector(spdiff_edist_vars[,,level,n,t])))
			ASD_PDomVar[level,n,t] <- sd(NaRV.omit(as.vector(spdiff_edist_vars[,,level,n,t])))
		}
	}
}

#---------------------------------------------------------------------------------------------------------------------
# Determine how the influence of variables on Euclidean distance diminishes from the most-influential to the least-influential variable

message("Determining influence decay of variables for SDPs")

TAvg_InflDecay <- array(0,c(nvar,nlevel,(nscen/2)))
TSD_InflDecay <- array(0,c(nvar,nlevel,(nscen/2)))

for(n in 1:(nscen/2))
{
	for(v in 1:nvar)
	{
		for(l in 1:nlevel)
		{
			TAvg_InflDecay[v,l,n] <- mean(Avg_PDomVar[v,l,n,])
			TSD_InflDecay[v,l,n] <- sd(Avg_PDomVar[v,l,n,])
		}
	}
}

#---------------------------------------------------------------------------------------------------------------------
# Find most influential variables for difference between transient decade and closest equilibrium decade

# do a permutation in which, one at a time, equilibrium and transient variable are set equal to one another, i.e., the difference between both = 0, and see how much the resulting Euclidean
# distance from the one calculated from the full set.

message("Identifying most influential variables for CDPs - takes a little while")

edist_perm_cross <- array(NA,c(nlon,nlat,nvar,(nscen/2),ntime))		# prozentuale Abweichung vom "vollen Wert" verursacht durch die betreffende Variable

for(dt in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		n1 <- n*2-1				# all uneven indices, i.e., all equilibrium scenarios
		n2 <- n*2				# all even indices, i.e., all transient scenarios
		
		for(x in 1:nlon)
		{
			for(y in 1:nlat)
			{
				if(!anyNA(std_vars[x,y, ,,dt]))
				{			
					for(v in 1:nvar)
					{
						t1_estd <- MinDec_EuPast[x,y,n,1,dt]		# equilibrium scenario: find the time index of the equilibrium year that goes together with the transient year's indey						
						estd_tmp <- std_vars[x,y, ,n1,t1_estd]		# estd_tmp: vector holding all 9 standardized vars of equilibrium scenario						
						estd_tmp[v] <- std_vars[x,y,v,n2,dt]		# replace the equilibrium variable under consideration with the transient variable under consideration, while leaving all other variables at their equilibrium values
						edist_perm_cross[x,y,v,n,dt]  <- (MinDec_EuPast[x,y,n,2,dt] - dist(rbind(estd_tmp, std_vars[x,y, ,n2,dt]), method="euclidean"))/MinDec_EuPast[x,y,n,2,dt] * 100. 				
					}
				}
			}
		}
	}	
}

#---------------------------------------------------------------------------------------------------------------------
# sort the variables by rank (as much as possible)

message("Sorting variables according to influence on full Euclidean distance beteen CDPs - takes a little while")

sort_edist_cvars <- array(NA,c(nlon,nlat,nvar,(nscen/2),ntime))		# sort variables in descending order according to the percent deviance its omittance causes from the "full" distance value
spdiff_edist_cvars <- array(NA,c(nlon,nlat,nvar,(nscen/2),ntime))

for(t in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		for(x in 1:nlon)
		{
			for(y in 1:nlat)
			{	
				if(!anyNA(edist_perm_cross[x,y, ,,t]))
				{
					for(r in 1:nvar)
						{
							idxa <- which(edist_perm_cross[x,y, ,n,t]==(sort(edist_perm_cross[x,y, ,n,t],TRUE)[r]))			# it may happen that some vars are equal in rank (usually the ones that = 0), therefore need this restriction
							if(length(idxa)==1) sort_edist_cvars[x,y,r,n,t] <- idxa
						}
				}
			}
		}
	}	
}

for(t in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		for(x in 1:nlon)
		{
			for(y in 1:nlat)
			 {
			 	for(v in 1:nvar)
			 	{
				 	idxs <- sort_edist_cvars[x,y,v,n,t]			# this is the variable that corresponds to the current rank, and the idx required to get the pdiff-value that goes with it
				 	if(!is.nan(idxs)) spdiff_edist_cvars[x,y,v,n,t] <- edist_perm_cross[x,y,idxs,n,t]
		 	}		 	
			 }
		}
	}	
}

#---------------------------------------------------------------------------------------------------------------------
# determine the number of grid cells that each variable "dominates" for a given level (not considering how strongly it dominates them in this step), for closest-decade pairing

message("Determining the number of grid cells dominated by each variable, for CDPs")

sum_dom_cvars <- array(0,c(nvar,(nscen/2),ntime))		# fraction of total African area "dominated" by a given variable, at a given time, for a given scenario
level <- 1

for(t in 1:ntime)
{
	for(n in 1:(nscen/2))
	{
		for(v in 1:nvar)
		{
			for(x in 1:nlon)
			{
				for(y in 1:nlat)
				{
					if(!is.na(sort_edist_cvars[x,y,level,n,t]))
					{
						if(sort_edist_cvars[x,y,level,n,t]==v) { sum_dom_cvars[v,n,t] <- sum_dom_cvars[v,n,t] + cellarea[x,y] }		
					}	
				}
			}
			sum_dom_cvars[v,n,t] <- sum_dom_cvars[v,n,t] / totarea[n,t] 	
		}
	}	
}

#---------------------------------------------------------------------------------------------------------------------
# variable contribution the percent deviance from total Euclidean distance for CDPs

message("Determining average variable contribution to full Euclidean distance, for CDPs")

nlevel <- nvar

Avg_PDomVar_cross <- array(0,c(nvar,nlevel,(nscen/2),ntime))
SD_PDomVar_cross <- array(0,c(nvar,nlevel,(nscen/2),ntime))
AAvg_PDomVar_cross <- array(0,c(nlevel,(nscen/2),ntime))
ASD_PDomVar_cross <- array(0,c(nlevel,(nscen/2),ntime))

for(level in 1:nvar)
{
	for(t in 1:ntime)
	{
		for(n in 1:(nscen/2))
		{
			for(v in 1:nvar)
			{
				h <- NULL				# make sure to re-set this helper for each var
				for(x in 1:nlon)
				{
					mask <- which(sort_edist_cvars[x,,level,n,t]==v)
					h <- c(h,spdiff_edist_cvars[x,mask,level,n,t])
				}
				Avg_PDomVar_cross[v,level,n,t] <- mean(h)
				SD_PDomVar_cross[v,level,n,t] <- sd(h)
			}
			AAvg_PDomVar_cross[level,n,t] <- mean(NaRV.omit(as.vector(spdiff_edist_cvars[,,level,n,t])))
			ASD_PDomVar_cross[level,n,t] <- sd(NaRV.omit(as.vector(spdiff_edist_cvars[,,level,n,t])))
				
		}
	}
}

#---------------------------------------------------------------------------------------------------------------------
# Determine how the influence of variables on Euclidean distance diminishes from the most-influential to the least-influential variable

message("Determining influence decay of variables for CDPs")

TAvg_InflDecay_cross <- array(0,c(nvar,nlevel,(nscen/2)))
TSD_InflDecay_cross <- array(0,c(nvar,nlevel,(nscen/2)))

for(n in 1:(nscen/2))
{
	for(v in 1:nvar)
	{
		for(l in 1:nlevel)
		{
			TAvg_InflDecay_cross[v,l,n] <- mean(Avg_PDomVar_cross[v,l,n,])
			TSD_InflDecay_cross[v,l,n] <- sd(Avg_PDomVar_cross[v,l,n,])
		}
	}
}

